#!/usr/bin/python3

# Script is designed to tune incoming VM xml-definition
# during migration from libvirt vz7 machine to libvirt vz9
# machine to avoid strict full CPU check that may lead to
# failure due to absence of rtl and hle features in hypervisor

import os
import sys
from lxml import etree

def feature_required(doc, feature_name):
    cpu = doc.xpath('/domain/cpu')
    if cpu == []:
        return False

    features = cpu[0].xpath('./feature')
    for feature in features:
        if feature.attrib['policy'] in ['require', 'force'] and \
           feature.attrib['name'] == feature_name:
            return True

    return False

def need_cpu_match_relax_intel(doc):
    # If we already have these features disabled,
    # no need to relax match rules
    if not feature_required(doc, 'hle') and \
       not feature_required(doc, 'rtm'):
        return False

    # If HCI version is >= 6 or we deal with another manufacturer
    # nothing should be done
    system = doc.xpath('/domain/sysinfo/system')
    if (system == [] or
        system[0].xpath('./entry[@name = "manufacturer"]') == [] or
        system[0].xpath('./entry[@name = "manufacturer"]')[0].text not in ['Acronis', 'Virtuozzo']):
        return False

    version = system[0].xpath('./entry[@name = "version"]')
    if version == []:
        return False

    version_tokens = version[0].text.split(".")
    if version_tokens != [] and version_tokens[0].isdigit():
        major = int(version_tokens[0])
        if major >= 6:
            return False

    return True

if len(sys.argv) < 3:
    print('Invalid number of arguments', file=sys.stderr)
    sys.exit(1)

op = sys.argv[2]
if op == 'migrate':
    parser = etree.XMLParser(remove_blank_text=True)
    doc = etree.parse(sys.stdin, parser)

    if not need_cpu_match_relax_intel(doc):
        sys.exit(0)

    cpu = doc.xpath('/domain/cpu')
    cpu[0].attrib['match'] = 'minimum'
    print(etree.tostring(doc, encoding='unicode', pretty_print=True))
